@@ -17,10 +17,7 @@
 */
 
 #include <Box2D/Collision/b2DynamicTree.h>
-#include <cstring>
-#include <cfloat>
-using namespace std;
-
+#include <string.h>
 
 b2DynamicTree::b2DynamicTree()
 {
@@ -769,3 +766,13 @@ void b2DynamicTree::RebuildBottomUp()
 
 	Validate();
 }
+
+void b2DynamicTree::ShiftOrigin(const b2Vec2& newOrigin)
+{
+	// Build array of leaves. Free the rest.
+	for (int32 i = 0; i < m_nodeCapacity; ++i)
+	{
+		m_nodes[i].aabb.lowerBound -= newOrigin;
+		m_nodes[i].aabb.upperBound -= newOrigin;
+	}
+}
