@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2019 LOVE Development Team
+ * Copyright (c) 2006-2023 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -61,6 +61,9 @@ public:
 	 **/
 	static const int DEFAULT_METER = 30;
 
+	Physics();
+	virtual ~Physics();
+
 	// Implements Module.
 	const char *getName() const;
 	virtual ModuleType getModuleType() const { return M_PHYSICS; }
