@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2019 LOVE Development Team
+ * Copyright (c) 2006-2023 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -46,6 +46,7 @@ public:
 	virtual ~LuaThread();
 	void threadFunction();
 	const std::string &getError() const;
+	bool hasError() const { return haserror; }
 
 	bool start(const std::vector<Variant> &args);
 
@@ -56,6 +57,7 @@ private:
 	StrongRef<love::Data> code;
 	std::string name;
 	std::string error;
+	bool haserror;
 
 	std::vector<Variant> args;
 
