@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2022 LOVE Development Team
+ * Copyright (c) 2006-2023 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -44,20 +44,28 @@ bool Thread::start()
 {
 #if defined(LOVE_LINUX)
 	// Temporarly block signals, as the thread inherits this mask
-	love::thread::disableSignals();
+	love::thread::ScopedDisableSignals disableSignals;
 #endif
 
 	Lock l(mutex);
+
 	if (running)
 		return false;
+
 	if (thread) // Clean old handle up
 		SDL_WaitThread(thread, nullptr);
+
+	// Keep the threadable around until the thread is done with it.
+	// This is done before thread_runner executes because there can be a delay
+	// between CreateThread and the start of the thread code's execution.
+	t->retain();
+
 	thread = SDL_CreateThread(thread_runner, t->getThreadName(), this);
 	running = (thread != nullptr);
 
-#if defined(LOVE_LINUX)
-	love::thread::reenableSignals();
-#endif
+	if (!running)
+		t->release(); // thread_runner is never called in this situation.
+
 	return running;
 }
 
@@ -83,7 +91,6 @@ bool Thread::isRunning()
 int Thread::thread_runner(void *data)
 {
 	Thread *self = (Thread *) data; // some compilers don't like 'this'
-	self->t->retain();
 
 	self->t->threadFunction();
 
@@ -92,6 +99,7 @@ int Thread::thread_runner(void *data)
 		self->running = false;
 	}
 
+	// This was retained in start().
 	self->t->release();
 	return 0;
 }
