@@ -1,5 +1,5 @@
 /**
- * Copyright (c) 2006-2022 LOVE Development Team
+ * Copyright (c) 2006-2023 LOVE Development Team
  *
  * This software is provided 'as-is', without any express or implied
  * warranty.  In no event will the authors be held liable for any damages
@@ -132,6 +132,12 @@ Thread *newThread(Threadable *t);
 #if defined(LOVE_LINUX)
 void disableSignals();
 void reenableSignals();
+
+struct ScopedDisableSignals
+{
+	ScopedDisableSignals() { disableSignals(); }
+	~ScopedDisableSignals() { reenableSignals(); }
+};
 #endif
 
 } // thread
